@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -52,7 +52,6 @@ class HipcubDeviceReduceTests : public ::testing::Test
 public:
     using input_type                        = typename Params::input_type;
     using output_type                       = typename Params::output_type;
-    static constexpr bool debug_synchronous = false;
 };
 
 typedef ::testing::Types<
@@ -99,7 +98,6 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
 
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
 
     const std::vector<size_t> sizes = get_sizes();
     for(auto size : sizes)
@@ -107,9 +105,18 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
+            if(test_utils::precision<U>::value * size > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                break;
+            }
+
             hipStream_t stream = 0; // default
 
             // Generate data
@@ -156,8 +163,7 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
                                        d_input,
                                        d_output,
                                        input.size(),
-                                       stream,
-                                       debug_synchronous);
+                                       stream);
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -172,8 +178,7 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
                                        d_input,
                                        d_output,
                                        input.size(),
-                                       stream,
-                                       debug_synchronous);
+                                       stream);
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -188,7 +193,10 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceSum)
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(output[0], expected, test_utils::precision_threshold<T>::percentage));
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(output[0],
+                                        expected,
+                                        test_utils::precision<U>::value * size));
 
             hipFree(d_input);
             hipFree(d_output);
@@ -205,7 +213,6 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
 
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
 
     const std::vector<size_t> sizes = get_sizes();
     for(auto size : sizes)
@@ -251,13 +258,12 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
             size_t temp_storage_size_bytes;
             void * d_temp_storage = nullptr;
             // Get size of d_temp_storage
-            HIP_CHECK(
-                hipcub::DeviceReduce::Min(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_input, d_output, input.size(),
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceReduce::Min(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                input.size(),
+                                                stream));
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -267,13 +273,12 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
             HIP_CHECK(hipDeviceSynchronize());
 
             // Run
-            HIP_CHECK(
-                hipcub::DeviceReduce::Min(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_input, d_output, input.size(),
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceReduce::Min(d_temp_storage,
+                                                temp_storage_size_bytes,
+                                                d_input,
+                                                d_output,
+                                                input.size(),
+                                                stream));
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -288,7 +293,12 @@ TYPED_TEST(HipcubDeviceReduceTests, ReduceMinimum)
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
-            ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(output[0], expected, 0.01f));
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(
+                output[0],
+                expected,
+                std::is_same<T, U>::value
+                    ? 0
+                    : std::max(test_utils::precision<T>::value, test_utils::precision<U>::value)));
 
             hipFree(d_input);
             hipFree(d_output);
@@ -305,16 +315,14 @@ struct ArgMinDispatch
                     InputIteratorT  d_in,
                     OutputIteratorT d_out,
                     int             num_items,
-                    hipStream_t     stream,
-                    bool            debug_synchronous) const
+                    hipStream_t     stream) const
     {
         return hipcub::DeviceReduce::ArgMin(d_temp_storage,
                                             temp_storage_bytes,
                                             d_in,
                                             d_out,
                                             num_items,
-                                            stream,
-                                            debug_synchronous);
+                                            stream);
     }
 };
 
@@ -326,16 +334,14 @@ struct ArgMaxDispatch
                     InputIteratorT  d_in,
                     OutputIteratorT d_out,
                     int             num_items,
-                    hipStream_t     stream,
-                    bool            debug_synchronous) const
+                    hipStream_t     stream) const
     {
         return hipcub::DeviceReduce::ArgMax(d_temp_storage,
                                             temp_storage_bytes,
                                             d_in,
                                             d_out,
                                             num_items,
-                                            stream,
-                                            debug_synchronous);
+                                            stream);
     }
 };
 
@@ -350,7 +356,6 @@ void test_argminmax(typename TestFixture::input_type empty_value)
     using Iterator  = typename hipcub::ArgIndexInputIterator<T*, int>;
     using key_value = typename Iterator::value_type;
 
-    const bool          debug_synchronous = TestFixture::debug_synchronous;
     DispatchFunction    function;
     std::vector<size_t> sizes = get_sizes();
     sizes.push_back(0);
@@ -400,8 +405,7 @@ void test_argminmax(typename TestFixture::input_type empty_value)
                                d_input,
                                d_output,
                                input.size(),
-                               stream,
-                               debug_synchronous));
+                               stream));
 
             // temp_storage_size_bytes must be > 0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -415,8 +419,7 @@ void test_argminmax(typename TestFixture::input_type empty_value)
                                d_input,
                                d_output,
                                input.size(),
-                               stream,
-                               debug_synchronous));
+                               stream));
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -431,7 +434,9 @@ void test_argminmax(typename TestFixture::input_type empty_value)
 
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output[0].key, expected.key));
             ASSERT_NO_FATAL_FAILURE(
-                test_utils::assert_near(output[0].value, expected.value, 0.01f));
+                test_utils::assert_near(output[0].value,
+                                        expected.value,
+                                        test_utils::precision<T>::value * size));
         }
     }
 }
@@ -472,7 +477,6 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     using Iterator  = typename hipcub::ArgIndexInputIterator<T*, int>;
     using key_value = typename Iterator::value_type;
 
-    constexpr bool   debug_synchronous = false;
     hipStream_t      stream            = 0; // default
     DispatchFunction function;
     constexpr size_t size = 100'000;
@@ -491,13 +495,8 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     size_t temp_storage_size_bytes{};
     void*  d_temp_storage{};
 
-    HIP_CHECK(function(d_temp_storage,
-                       temp_storage_size_bytes,
-                       d_input,
-                       d_output,
-                       input.size(),
-                       stream,
-                       debug_synchronous));
+    HIP_CHECK(
+        function(d_temp_storage, temp_storage_size_bytes, d_input, d_output, input.size(), stream));
 
     // temp_storage_size_bytes must be > 0
     ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -505,13 +504,8 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
     HIP_CHECK(hipDeviceSynchronize());
 
-    HIP_CHECK(function(d_temp_storage,
-                       temp_storage_size_bytes,
-                       d_input,
-                       d_output,
-                       input.size(),
-                       stream,
-                       debug_synchronous));
+    HIP_CHECK(
+        function(d_temp_storage, temp_storage_size_bytes, d_input, d_output, input.size(), stream));
     HIP_CHECK(hipPeekAtLastError());
     HIP_CHECK(hipDeviceSynchronize());
 
@@ -567,7 +561,6 @@ class HipcubDeviceReduceLargeIndicesTests : public ::testing::Test
 public:
     using input_type                        = typename Params::input_type;
     using output_type                       = typename Params::output_type;
-    static constexpr bool debug_synchronous = false;
 };
 
 typedef ::testing::Types<DeviceReduceParams<short, size_t>,
@@ -588,7 +581,6 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
 
     using T                      = typename TestFixture::input_type;
     using U                      = typename TestFixture::output_type;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
     using IteratorType           = hipcub::ConstantInputIterator<T>;
 
     const std::vector<size_t> exponents = {30, 31, 32, 33, 34};
@@ -612,9 +604,6 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_output, output.size() * sizeof(U)));
             HIP_CHECK(hipDeviceSynchronize());
 
-            // Calculate expected results on host
-            const U expected = static_cast<U>(size);
-
             // Temp storage
             size_t temp_storage_size_bytes;
             void*  d_temp_storage = nullptr;
@@ -625,8 +614,7 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
                                       d_input,
                                       d_output,
                                       size,
-                                      stream,
-                                      debug_synchronous);
+                                      stream);
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -641,8 +629,7 @@ TYPED_TEST(HipcubDeviceReduceLargeIndicesTests, LargeIndices)
                                       d_input,
                                       d_output,
                                       size,
-                                      stream,
-                                      debug_synchronous);
+                                      stream);
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
