@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,12 @@
 #include "common_test_header.hpp"
 
 // Thread operators fixes for extended float types
+#include "test_utils_data_generation.hpp"
 #include "test_utils_thread_operators.hpp"
 
 // hipcub API
 #include "hipcub/device/device_segmented_reduce.hpp"
-#include "test_utils_data_generation.hpp"
+#include "hipcub/iterator/counting_input_iterator.hpp"
 
 std::vector<size_t> get_sizes()
 {
@@ -67,19 +68,16 @@ public:
     using params = Params;
 };
 
-typedef ::testing::Types<params1<unsigned int, unsigned int, hipcub::Sum>,
-                         params1<int, int, hipcub::Sum, -100, 0, 10000>,
-                         params1<double, double, hipcub::Min, 1000, 0, 10000>,
-                         params1<int, short, hipcub::Max, 10, 1000, 10000>,
-                         params1<short, double, hipcub::Sum, 5, 1, 1000>,
-                         params1<float, double, hipcub::Max, 50, 2, 10>,
-                         params1<float, float, hipcub::Sum, 123, 100, 200>
-#ifdef __HIP_PLATFORM_AMD__
-                         ,
-                         params1<test_utils::half, test_utils::half, hipcub::Max, 50, 2, 10>,
-                         params1<test_utils::bfloat16, test_utils::bfloat16, hipcub::Max, 50, 2, 10>
-#endif
-                         >
+typedef ::testing::Types<
+    params1<unsigned int, unsigned int, hipcub::Sum>,
+    params1<int, int, hipcub::Sum, -100, 0, 10000>,
+    params1<double, double, hipcub::Min, 1000, 0, 10000>,
+    params1<int, short, hipcub::Max, 10, 1000, 10000>,
+    params1<short, double, hipcub::Sum, 5, 1, 1000>,
+    params1<float, double, hipcub::Max, 50, 2, 10>,
+    params1<float, float, hipcub::Sum, 123, 100, 200>,
+    params1<test_utils::half, test_utils::half, hipcub::Max, 50, 2, 10>,
+    params1<test_utils::bfloat16, test_utils::bfloat16, hipcub::Max, 50, 2, 10>>
     Params1;
 
 TYPED_TEST_SUITE(HipcubDeviceSegmentedReduceOp, Params1);
@@ -98,7 +96,6 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
     using offset_type = unsigned int;
 
     const input_type init = test_utils::convert_to_device<input_type>(TestFixture::params::init);
-    const bool debug_synchronous = false;
     reduce_op_type reduce_op;
 
     std::random_device rd;
@@ -115,7 +112,7 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             hipStream_t stream = 0; // default
@@ -131,18 +128,21 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
             );
 
             std::vector<offset_type> offsets;
-            unsigned int segments_count = 0;
-            size_t offset = 0;
+            unsigned int             segments_count     = 0;
+            size_t                   offset             = 0;
+            size_t                   max_segment_length = 0;
             while(offset < size)
             {
                 const size_t segment_length = segment_length_dis(gen);
                 offsets.push_back(offset);
 
-                const size_t end = std::min(size, offset + segment_length);
+                const size_t end   = std::min(size, offset + segment_length);
+                max_segment_length = std::max(max_segment_length, end - offset);
+
                 result_type aggregate = init;
                 for(size_t i = offset; i < end; i++)
                 {
-                    aggregate = reduce_op(aggregate, static_cast<result_type>(values_input[i]));
+                    aggregate = reduce_op(aggregate, values_input[i]);
                 }
                 aggregates_expected.push_back(aggregate);
 
@@ -151,6 +151,16 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
             }
             offsets.push_back(size);
 
+            const float precision = test_utils::precision<result_type>::value * max_segment_length;
+            if(precision > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                continue;
+            }
+
             input_type * d_values_input;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_input, size * sizeof(input_type)));
             HIP_CHECK(
@@ -176,32 +186,32 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
 
             size_t temporary_storage_bytes;
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Reduce(
-                    nullptr, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    reduce_op, init,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Reduce(nullptr,
+                                                            temporary_storage_bytes,
+                                                            d_values_input,
+                                                            d_aggregates_output,
+                                                            segments_count,
+                                                            d_offsets,
+                                                            d_offsets + 1,
+                                                            reduce_op,
+                                                            init,
+                                                            stream));
 
             ASSERT_GT(temporary_storage_bytes, 0U);
 
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Reduce(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    reduce_op, init,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Reduce(d_temporary_storage,
+                                                            temporary_storage_bytes,
+                                                            d_values_input,
+                                                            d_aggregates_output,
+                                                            segments_count,
+                                                            d_offsets,
+                                                            d_offsets + 1,
+                                                            reduce_op,
+                                                            init,
+                                                            stream));
 
             HIP_CHECK(hipFree(d_temporary_storage));
 
@@ -218,23 +228,8 @@ TYPED_TEST(HipcubDeviceSegmentedReduceOp, Reduce)
             HIP_CHECK(hipFree(d_offsets));
             HIP_CHECK(hipFree(d_aggregates_output));
 
-            for(size_t i = 0; i < segments_count; i++)
-            {
-                if(std::is_integral<output_type>::value)
-                {
-                    ASSERT_EQ(test_utils::convert_to_native(aggregates_output[i]),
-                              test_utils::convert_to_native(aggregates_expected[i]));
-                }
-                else
-                {
-                    auto diff = std::max<test_utils::convert_to_fundamental_t<output_type>>(
-                        std::abs(0.01 * test_utils::convert_to_fundamental(aggregates_expected[i])),
-                        output_type(0.01));
-                    ASSERT_NEAR(test_utils::convert_to_native(aggregates_output[i]),
-                                test_utils::convert_to_native(aggregates_expected[i]),
-                                diff);
-                }
-            }
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(aggregates_output, aggregates_expected, precision));
         }
     }
 }
@@ -287,8 +282,7 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
     using result_type = output_type;
     using offset_type = unsigned int;
 
-    const input_type init              = input_type(0);
-    const bool       debug_synchronous = false;
+    const input_type init = input_type(0);
     reduce_op_type   reduce_op;
 
     std::random_device rd;
@@ -305,7 +299,7 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             hipStream_t stream = 0; // default
@@ -321,14 +315,17 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
             );
 
             std::vector<offset_type> offsets;
-            unsigned int segments_count = 0;
-            size_t offset = 0;
+            unsigned int             segments_count     = 0;
+            size_t                   offset             = 0;
+            size_t                   max_segment_length = 0;
             while(offset < size)
             {
                 const size_t segment_length = segment_length_dis(gen);
                 offsets.push_back(offset);
 
-                const size_t end = std::min(size, offset + segment_length);
+                const size_t end   = std::min(size, offset + segment_length);
+                max_segment_length = std::max(max_segment_length, end - offset);
+
                 result_type aggregate = init;
                 for(size_t i = offset; i < end; i++)
                 {
@@ -341,6 +338,16 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
             }
             offsets.push_back(size);
 
+            const float precision = test_utils::precision<result_type>::value * max_segment_length;
+            if(precision > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                continue;
+            }
+
             input_type * d_values_input;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_input, size * sizeof(input_type)));
             HIP_CHECK(
@@ -366,30 +373,28 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
 
             size_t temporary_storage_bytes;
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Sum(
-                    nullptr, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Sum(nullptr,
+                                                         temporary_storage_bytes,
+                                                         d_values_input,
+                                                         d_aggregates_output,
+                                                         segments_count,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
 
             ASSERT_GT(temporary_storage_bytes, 0U);
 
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Sum(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Sum(d_temporary_storage,
+                                                         temporary_storage_bytes,
+                                                         d_values_input,
+                                                         d_aggregates_output,
+                                                         segments_count,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
 
             HIP_CHECK(hipFree(d_temporary_storage));
 
@@ -406,22 +411,8 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Sum)
             HIP_CHECK(hipFree(d_offsets));
             HIP_CHECK(hipFree(d_aggregates_output));
 
-            for(size_t i = 0; i < segments_count; i++)
-            {
-                if(std::is_integral<output_type>::value)
-                {
-                    ASSERT_EQ(aggregates_output[i], aggregates_expected[i]);
-                }
-                else
-                {
-                    auto diff = std::max<output_type>(
-                        std::abs(0.01 * aggregates_expected[i]), output_type(0.01)
-                    );
-                    ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(aggregates_output[i],
-                                                                    aggregates_expected[i],
-                                                                    diff));
-                }
-            }
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(aggregates_output, aggregates_expected, precision));
         }
     }
 }
@@ -439,7 +430,6 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
     using offset_type = unsigned int;
 
     constexpr input_type init = std::numeric_limits<input_type>::max();
-    const bool debug_synchronous = false;
     reduce_op_type reduce_op;
 
     std::random_device rd;
@@ -456,7 +446,7 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
         for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
-            SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+            SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
             hipStream_t stream = 0; // default
@@ -472,14 +462,17 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
             );
 
             std::vector<offset_type> offsets;
-            unsigned int segments_count = 0;
-            size_t offset = 0;
+            unsigned int             segments_count     = 0;
+            size_t                   offset             = 0;
+            size_t                   max_segment_length = 0;
             while(offset < size)
             {
                 const size_t segment_length = segment_length_dis(gen);
                 offsets.push_back(offset);
 
-                const size_t end = std::min(size, offset + segment_length);
+                const size_t end   = std::min(size, offset + segment_length);
+                max_segment_length = std::max(max_segment_length, end - offset);
+
                 result_type aggregate = init;
                 for(size_t i = offset; i < end; i++)
                 {
@@ -492,6 +485,16 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
             }
             offsets.push_back(size);
 
+            const float precision = test_utils::precision<result_type>::value * max_segment_length;
+            if(precision > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                continue;
+            }
+
             input_type * d_values_input;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_input, size * sizeof(input_type)));
             HIP_CHECK(
@@ -517,30 +520,28 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
 
             size_t temporary_storage_bytes;
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Min(
-                    nullptr, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Min(nullptr,
+                                                         temporary_storage_bytes,
+                                                         d_values_input,
+                                                         d_aggregates_output,
+                                                         segments_count,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
 
             ASSERT_GT(temporary_storage_bytes, 0U);
 
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                hipcub::DeviceSegmentedReduce::Min(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_values_input, d_aggregates_output,
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Min(d_temporary_storage,
+                                                         temporary_storage_bytes,
+                                                         d_values_input,
+                                                         d_aggregates_output,
+                                                         segments_count,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
 
             HIP_CHECK(hipFree(d_temporary_storage));
 
@@ -557,11 +558,8 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, Min)
             HIP_CHECK(hipFree(d_offsets));
             HIP_CHECK(hipFree(d_aggregates_output));
 
-            for(size_t i = 0; i < segments_count; i++)
-            {
-                ASSERT_NO_FATAL_FAILURE(
-                    test_utils::assert_near(aggregates_output[i], aggregates_expected[i], .01f));
-            }
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(aggregates_output, aggregates_expected, precision));
         }
     }
 }
@@ -576,8 +574,7 @@ struct ArgMinDispatch
                     int             num_segments,
                     OffsetIteratorT d_begin_offsets,
                     OffsetIteratorT d_end_offsets,
-                    hipStream_t     stream,
-                    bool            debug_synchronous) const
+                    hipStream_t     stream) const
     {
         return hipcub::DeviceSegmentedReduce::ArgMin(d_temp_storage,
                                                      temp_storage_bytes,
@@ -586,8 +583,7 @@ struct ArgMinDispatch
                                                      num_segments,
                                                      d_begin_offsets,
                                                      d_end_offsets,
-                                                     stream,
-                                                     debug_synchronous);
+                                                     stream);
     }
 };
 
@@ -601,8 +597,7 @@ struct ArgMaxDispatch
                     int             num_segments,
                     OffsetIteratorT d_begin_offsets,
                     OffsetIteratorT d_end_offsets,
-                    hipStream_t     stream,
-                    bool            debug_synchronous) const
+                    hipStream_t     stream) const
     {
         return hipcub::DeviceSegmentedReduce::ArgMax(d_temp_storage,
                                                      temp_storage_bytes,
@@ -611,8 +606,7 @@ struct ArgMaxDispatch
                                                      num_segments,
                                                      d_begin_offsets,
                                                      d_end_offsets,
-                                                     stream,
-                                                     debug_synchronous);
+                                                     stream);
     }
 };
 
@@ -629,7 +623,6 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
     using offset_type = unsigned int;
 
     DispatchFunction                      function;
-    const bool                            debug_synchronous = false;
     std::random_device                    rd;
     std::default_random_engine            gen(rd());
     std::uniform_int_distribution<size_t> segment_length_dis(
@@ -656,15 +649,17 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
 
             HostOp                   host_op{};
             std::vector<offset_type> offsets;
-            unsigned int             segments_count = 0;
-            size_t                   offset         = 0;
+            unsigned int             segments_count     = 0;
+            size_t                   offset             = 0;
+            size_t                   max_segment_length = 0;
             while(offset < size)
             {
                 const size_t segment_length = segment_length_dis(gen);
                 offsets.push_back(offset);
                 Iterator x(&values_input[offset]);
 
-                const size_t end = std::min(size, offset + segment_length);
+                const size_t end   = std::min(size, offset + segment_length);
+                max_segment_length = std::max(max_segment_length, end - offset);
                 if(offset < end)
                 {
                     key_value aggregate(0, values_input[offset]);
@@ -686,6 +681,16 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
             }
             offsets.push_back(size);
 
+            const float precision = test_utils::precision<key_value>::value * max_segment_length;
+            if(precision > 0.5)
+            {
+                std::cout << "Test is skipped from size " << size
+                          << " on, potential error of summation is more than 0.5 of the result "
+                             "with current or larger size"
+                          << std::endl;
+                continue;
+            }
+
             input_type* d_values_input;
             HIP_CHECK(
                 test_common_utils::hipMallocHelper(&d_values_input, size * sizeof(input_type)));
@@ -716,8 +721,7 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
                                segments_count,
                                d_offsets,
                                d_offsets + 1,
-                               stream,
-                               debug_synchronous));
+                               stream));
 
             // temp_storage_size_bytes must be > 0
             ASSERT_GT(temporary_storage_bytes, 0U);
@@ -733,8 +737,7 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
                                segments_count,
                                d_offsets,
                                d_offsets + 1,
-                               stream,
-                               debug_synchronous));
+                               stream));
             HIP_CHECK(hipPeekAtLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -755,7 +758,7 @@ void test_argminmax(typename TestFixture::params::input_type empty_value)
                     test_utils::assert_eq(aggregates_output[i].key, aggregates_expected[i].key));
                 ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(aggregates_output[i].value,
                                                                 aggregates_expected[i].value,
-                                                                .01f));
+                                                                precision));
             }
         }
     }
@@ -778,13 +781,13 @@ TYPED_TEST(HipcubDeviceSegmentedReduce, ArgMax)
 }
 
 template<class T>
-class HipcubDeviceReduceArgMinMaxSpecialTests : public testing::Test
+class HipcubDeviceSegmentedReduceArgMinMaxSpecialTests : public testing::Test
 {};
 
-using HipcubDeviceReduceArgMinMaxSpecialTestsParams
+using HipcubDeviceSegmentedReduceArgMinMaxSpecialTestsParams
     = ::testing::Types<float, test_utils::half, test_utils::bfloat16>;
-TYPED_TEST_SUITE(HipcubDeviceReduceArgMinMaxSpecialTests,
-                 HipcubDeviceReduceArgMinMaxSpecialTestsParams);
+TYPED_TEST_SUITE(HipcubDeviceSegmentedReduceArgMinMaxSpecialTests,
+                 HipcubDeviceSegmentedReduceArgMinMaxSpecialTestsParams);
 
 template<typename TypeParam, typename DispatchFunction>
 void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
@@ -798,7 +801,6 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
     using key_value   = typename Iterator::value_type;
     using offset_type = unsigned int;
 
-    constexpr bool             debug_synchronous = false;
     hipStream_t                stream            = 0; // default
     DispatchFunction           function;
     std::random_device         rd;
@@ -857,8 +859,7 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
                            segments_count,
                            d_offsets,
                            d_offsets + 1,
-                           stream,
-                           debug_synchronous));
+                           stream));
 
         // temp_storage_size_bytes must be > 0
         ASSERT_GT(temporary_storage_bytes, 0U);
@@ -874,8 +875,7 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
                            segments_count,
                            d_offsets,
                            d_offsets + 1,
-                           stream,
-                           debug_synchronous));
+                           stream));
 
         std::vector<key_value> aggregates_output(segments_count);
         HIP_CHECK(hipMemcpy(aggregates_output.data(),
@@ -910,7 +910,7 @@ void test_argminmax_allinf(TypeParam value, TypeParam empty_value)
 // TODO: enable for NVIDIA platform once CUB backend incorporates fix
 #ifdef __HIP_PLATFORM_AMD__
 /// ArgMin with all +Inf should result in +Inf.
-TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMinInf)
+TYPED_TEST(HipcubDeviceSegmentedReduceArgMinMaxSpecialTests, ReduceArgMinInf)
 {
     test_argminmax_allinf<TypeParam, ArgMinDispatch>(
         test_utils::numeric_limits<TypeParam>::infinity(),
@@ -918,10 +918,139 @@ TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMinInf)
 }
 
 /// ArgMax with all -Inf should result in -Inf.
-TYPED_TEST(HipcubDeviceReduceArgMinMaxSpecialTests, ReduceArgMaxInf)
+TYPED_TEST(HipcubDeviceSegmentedReduceArgMinMaxSpecialTests, ReduceArgMaxInf)
 {
     test_argminmax_allinf<TypeParam, ArgMaxDispatch>(
         test_utils::numeric_limits<TypeParam>::infinity_neg(),
         test_utils::numeric_limits<TypeParam>::lowest());
 }
 #endif // __HIP_PLATFORM_AMD__
+
+// ---------------------------------------------------------
+// Test for large indices
+// ---------------------------------------------------------
+
+TEST(HipcubDeviceSegmentedReduceLargeIndicesTests, LargeIndices)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id= " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using T              = size_t;
+    using input_type     = T;
+    using output_type    = T;
+    using IteratorType   = hipcub::CountingInputIterator<input_type>;
+    using reduce_op_type = typename hipcub::Sum;
+    using offset_type    = T;
+
+    const input_type init = input_type(0);
+    reduce_op_type   reduce_op;
+
+    static constexpr hipStream_t stream = 0; // default
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
+
+        const std::vector<size_t> sizes = test_utils::get_large_sizes(seed_value);
+
+        for(const auto size : sizes)
+        {
+            SCOPED_TRACE(testing::Message() << "with size = " << size);
+
+            // Generate data
+            const size_t min_segment_length = size_t{1} << 31;
+            const size_t max_segment_length = std::max(min_segment_length, size);
+
+            std::random_device                    rd;
+            std::default_random_engine            gen(seed_value);
+            std::uniform_int_distribution<size_t> segment_length_dis(min_segment_length,
+                                                                     max_segment_length);
+
+            const auto gauss_sum = [&](output_type n)
+            { return (n % 2 == 0) ? (n / 2) * (n - 1) : n * ((n - 1) / 2); };
+
+            std::vector<output_type> aggregates_expected;
+            std::vector<offset_type> offsets;
+
+            unsigned int num_segments = 0;
+            offset_type  offset       = 0;
+            while(offset < size)
+            {
+                const size_t segment_length = segment_length_dis(gen);
+                offsets.push_back(offset);
+
+                const offset_type end       = std::min(size, offset + segment_length);
+                output_type       aggregate = init;
+                aggregate = reduce_op(aggregate, gauss_sum(end) - gauss_sum(offset));
+                aggregates_expected.push_back(aggregate);
+
+                num_segments++;
+                offset += segment_length;
+            }
+            offsets.push_back(size);
+
+            // Device inputs
+            IteratorType d_input(input_type{0});
+            offset_type* d_offsets = nullptr;
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_offsets,
+                                                         sizeof(offset_type) * (num_segments + 1)));
+            HIP_CHECK(hipMemcpy(d_offsets,
+                                offsets.data(),
+                                sizeof(offset_type) * (num_segments + 1),
+                                hipMemcpyHostToDevice));
+
+            // Device outputs
+            output_type* d_aggregates_output;
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_aggregates_output,
+                                                         sizeof(output_type) * num_segments));
+            HIP_CHECK(hipDeviceSynchronize());
+
+            // Temp storage
+            size_t temp_storage_size_bytes;
+            void*  d_temp_storage = nullptr;
+
+            // Get size of d_temp_storage
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Sum(d_temp_storage,
+                                                         temp_storage_size_bytes,
+                                                         d_input,
+                                                         d_aggregates_output,
+                                                         num_segments,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
+
+            // temp_storage_size_bytes must be >0
+            ASSERT_GT(temp_storage_size_bytes, 0U);
+
+            // Allocate temporary storage
+            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+
+            // Run
+            HIP_CHECK(hipcub::DeviceSegmentedReduce::Sum(d_temp_storage,
+                                                         temp_storage_size_bytes,
+                                                         d_input,
+                                                         d_aggregates_output,
+                                                         num_segments,
+                                                         d_offsets,
+                                                         d_offsets + 1,
+                                                         stream));
+
+            // Copy output to host
+            std::vector<output_type> aggregates_output(num_segments);
+            HIP_CHECK(hipMemcpy(aggregates_output.data(),
+                                d_aggregates_output,
+                                sizeof(output_type) * num_segments,
+                                hipMemcpyDeviceToHost));
+
+            HIP_CHECK(hipFree(d_offsets));
+            HIP_CHECK(hipFree(d_temp_storage));
+            HIP_CHECK(hipFree(d_aggregates_output));
+
+            // Check if output values are as expected
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(aggregates_output, aggregates_expected));
+        }
+    }
+}
