@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -245,8 +245,7 @@ inline void sort_keys()
     constexpr bool descending = TestFixture::params::descending;
     constexpr unsigned int min_segment_length = TestFixture::params::min_segment_length;
     constexpr unsigned int max_segment_length = TestFixture::params::max_segment_length;
-    constexpr hipStream_t stream = 0;
-    constexpr bool debug_synchronous = false;
+    constexpr hipStream_t  stream             = 0;
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; ++seed_index)
     {
@@ -284,12 +283,16 @@ inline void sort_keys()
             void * d_temporary_storage{};
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            dispatch_sort_keys(
-                method, d_temporary_storage, temporary_storage_bytes,
-                d_keys_input, d_keys_output, size,
-                segments_count, d_offsets, d_offsets + 1,
-                stream, debug_synchronous
-            );
+            dispatch_sort_keys(method,
+                               d_temporary_storage,
+                               temporary_storage_bytes,
+                               d_keys_input,
+                               d_keys_output,
+                               size,
+                               segments_count,
+                               d_offsets,
+                               d_offsets + 1,
+                               stream);
 
             const std::vector<key_type> keys_output = download(d_keys_output, size);
 
@@ -316,8 +319,7 @@ inline void sort_keys_double_buffer()
     constexpr bool descending = TestFixture::params::descending;
     constexpr unsigned int min_segment_length = TestFixture::params::min_segment_length;
     constexpr unsigned int max_segment_length = TestFixture::params::max_segment_length;
-    constexpr hipStream_t stream = 0;
-    constexpr bool debug_synchronous = false;
+    constexpr hipStream_t  stream             = 0;
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; ++seed_index)
     {
@@ -357,12 +359,15 @@ inline void sort_keys_double_buffer()
             void * d_temporary_storage{};
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            dispatch_sort_keys(
-                method, d_temporary_storage, temporary_storage_bytes,
-                d_keys, size,
-                segments_count, d_offsets, d_offsets + 1,
-                stream, debug_synchronous
-            );
+            dispatch_sort_keys(method,
+                               d_temporary_storage,
+                               temporary_storage_bytes,
+                               d_keys,
+                               size,
+                               segments_count,
+                               d_offsets,
+                               d_offsets + 1,
+                               stream);
 
             const std::vector<key_type> keys_output = download(d_keys.Current(), size);
 
@@ -420,8 +425,7 @@ inline void sort_pairs()
     constexpr bool descending = TestFixture::params::descending;
     constexpr unsigned int min_segment_length = TestFixture::params::min_segment_length;
     constexpr unsigned int max_segment_length = TestFixture::params::max_segment_length;
-    constexpr hipStream_t stream = 0;
-    constexpr bool debug_synchronous = false;
+    constexpr hipStream_t  stream             = 0;
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; ++seed_index)
     {
@@ -466,14 +470,18 @@ inline void sort_pairs()
             void * d_temporary_storage{};
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            dispatch_sort_pairs(
-                method, d_temporary_storage, temporary_storage_bytes,
-                d_keys_input, d_keys_output,
-                d_values_input, d_values_output,
-                size, segments_count,
-                d_offsets, d_offsets + 1,
-                stream, debug_synchronous
-            );
+            dispatch_sort_pairs(method,
+                                d_temporary_storage,
+                                temporary_storage_bytes,
+                                d_keys_input,
+                                d_keys_output,
+                                d_values_input,
+                                d_values_output,
+                                size,
+                                segments_count,
+                                d_offsets,
+                                d_offsets + 1,
+                                stream);
 
             const std::vector<key_type> keys_output = download(d_keys_output, size);
             const std::vector<value_type> values_output = download(d_values_output, size);
@@ -508,8 +516,7 @@ inline void sort_pairs_double_buffer()
     constexpr bool descending = TestFixture::params::descending;
     constexpr unsigned int min_segment_length = TestFixture::params::min_segment_length;
     constexpr unsigned int max_segment_length = TestFixture::params::max_segment_length;
-    constexpr hipStream_t stream = 0;
-    constexpr bool debug_synchronous = false;
+    constexpr hipStream_t  stream             = 0;
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; ++seed_index)
     {
@@ -556,14 +563,17 @@ inline void sort_pairs_double_buffer()
             void * d_temporary_storage{};
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            dispatch_sort_pairs(
-                method, d_temporary_storage, temporary_storage_bytes,
-                d_keys, d_values,
-                size, segments_count,
-                d_offsets, d_offsets + 1,
-                stream, debug_synchronous
-            );
-            
+            dispatch_sort_pairs(method,
+                                d_temporary_storage,
+                                temporary_storage_bytes,
+                                d_keys,
+                                d_values,
+                                size,
+                                segments_count,
+                                d_offsets,
+                                d_offsets + 1,
+                                stream);
+
             const std::vector<key_type> keys_output = download(d_keys.Current(), size);
             const std::vector<value_type> values_output = download(d_values.Current(), size);
 
