@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2023-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -385,8 +385,7 @@ void scan_op_test(std::vector<InputT>  h_input,
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    hipStream_t stream            = 0;
-    const bool  debug_synchronous = false;
+    hipStream_t stream = 0;
 
     // Allocate input and output on device and copy input from host.
     InputT*  d_input{};
@@ -405,8 +404,7 @@ void scan_op_test(std::vector<InputT>  h_input,
                                                 d_output,
                                                 op,
                                                 input_size,
-                                                stream,
-                                                debug_synchronous));
+                                                stream));
 
     // Size of temporary storage must be > 0.
     ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -421,8 +419,7 @@ void scan_op_test(std::vector<InputT>  h_input,
                                                 d_output,
                                                 op,
                                                 input_size,
-                                                stream,
-                                                debug_synchronous));
+                                                stream));
     HIP_CHECK(hipGetLastError());
 
     // Copy output to host.
@@ -431,13 +428,10 @@ void scan_op_test(std::vector<InputT>  h_input,
         hipMemcpy(h_output.data(), d_output, input_size * sizeof(OutputT), hipMemcpyDeviceToHost));
 
     // Check output.
-    for(size_t i = 0; i < input_size; ++i)
-    {
-        auto tolerance = std::max<OutputT>(std::abs(0.1f * h_expected[i]), OutputT(0.01f));
-        if(std::is_integral<OutputT>::value)
-            tolerance = 0;
-        ASSERT_NEAR(h_output[i], h_expected[i], tolerance) << "where index = " << i;
-    }
+    ASSERT_NO_FATAL_FAILURE(
+        test_utils::assert_near(h_output,
+                                h_expected,
+                                test_utils::precision<InputT>::value * input_size));
 
     // Check output type.
     for(size_t i = 0; i < input_size; ++i)
@@ -550,14 +544,14 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, ReduceBySegmentOp)
         for(size_t i = 0; i < segment_count; i++)
         {
             // Check keys.
-            ASSERT_EQ(expected[i].key, output[i].key) << "where index = " << i;
+            ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(expected[i].key, output[i].key))
+                << "where index = " << i;
 
             // Check values.
-            auto tolerance
-                = std::max<output_type>(std::abs(0.1f * expected[i].value), output_type(0.01f));
-            if(std::is_integral<output_type>::value)
-                tolerance = 0;
-            ASSERT_NEAR(expected[i].value, output[i].value, tolerance) << "where index = " << i;
+            auto tolerance = test_utils::precision<input_type>::value * i;
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(expected[i].value, output[i].value, tolerance))
+                << "where index = " << i;
         }
     }
 }
@@ -574,8 +568,7 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, ReduceByKeyOp)
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
 
-    hipStream_t stream            = 0;
-    const bool  debug_synchronous = false;
+    hipStream_t stream = 0;
 
     const std::vector<size_t> sizes = get_sizes();
     for(auto input_size : sizes)
@@ -646,8 +639,7 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, ReduceByKeyOp)
                                                     d_unique_keys_expected,
                                                     sum_op,
                                                     input_size,
-                                                    stream,
-                                                    debug_synchronous));
+                                                    stream));
 
         // Size of temporary storage must be > 0.
         ASSERT_GT(temp_storage_size_bytes, 0U);
@@ -665,8 +657,7 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, ReduceByKeyOp)
                                                     d_unique_keys_expected,
                                                     sum_op,
                                                     input_size,
-                                                    stream,
-                                                    debug_synchronous));
+                                                    stream));
         HIP_CHECK(hipGetLastError());
 
         // Copy expected results to host.
@@ -691,18 +682,12 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, ReduceByKeyOp)
         // Check if output values are as expected.
         // Check number of unique keys.
         ASSERT_EQ(h_unique_keys_expected[0], h_unique_keys);
-        for(size_t i = 0; i < h_unique_keys; ++i)
-        {
-            // Check keys.
-            ASSERT_EQ(h_keys_expected[i], h_keys_output[i]) << "where index = " << i;
 
-            // Check values.
-            auto tolerance
-                = std::max<output_type>(std::abs(0.1f * h_expected[i]), output_type(0.01f));
-            if(std::is_integral<output_type>::value)
-                tolerance = 0;
-            ASSERT_NEAR(h_expected[i], h_output[i], tolerance) << "where index = " << i;
-        }
+        ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(h_keys_expected, h_keys_output));
+        ASSERT_NO_FATAL_FAILURE(
+            test_utils::assert_near(h_expected,
+                                    h_output,
+                                    test_utils::precision<input_type>::value * input_size));
 
         // Free resources.
         HIP_CHECK(hipFree(d_input));
@@ -779,11 +764,10 @@ TYPED_TEST(HipcubNCThreadOperatorsTests, CastOp)
         // Check output.
         for(size_t i = 0; i < input_size; ++i)
         {
-            auto tolerance
-                = std::max<output_type>(std::abs(0.1f * expected[i]), output_type(0.01f));
-            if(std::is_integral<output_type>::value)
-                tolerance = 0;
-            ASSERT_NEAR(output[i], expected[i], tolerance);
+            ASSERT_NO_FATAL_FAILURE(
+                test_utils::assert_near(output[i],
+                                        expected[i],
+                                        test_utils::precision<input_type>::value));
         }
 
         // Check output type.
