@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -63,6 +63,27 @@ typedef ::testing::Types<
 #ifdef __HIP_PLATFORM_AMD__
     params<float, 64U>,
 #endif
+    // half
+    params<test_utils::half, 1U>,
+    params<test_utils::half, 2U>,
+    params<test_utils::half, 4U>,
+    params<test_utils::half, 8U>,
+    params<test_utils::half, 16U>,
+    params<test_utils::half, 32U>,
+#ifdef __HIP_PLATFORM_AMD__
+    params<test_utils::half, 64U>,
+#endif
+    // bfloat16
+    params<test_utils::bfloat16, 1U>,
+    params<test_utils::bfloat16, 2U>,
+    params<test_utils::bfloat16, 4U>,
+    params<test_utils::bfloat16, 8U>,
+    params<test_utils::bfloat16, 16U>,
+    params<test_utils::bfloat16, 32U>,
+#ifdef __HIP_PLATFORM_AMD__
+    params<test_utils::bfloat16, 64U>,
+#endif
+
     // shared memory reduce
     // Integer
     params<int, 3U>,
@@ -120,6 +141,11 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
 
@@ -162,20 +188,20 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
         // Generate data
-        std::vector<T> input = test_utils::get_random_data<T>(size, -100, 100, seed_value);
-        std::vector<T> output(size / logical_warp_size, 0);
-        std::vector<T> expected(output.size(), 1);
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size / logical_warp_size);
+        std::vector<T> expected(output.size());
 
         // Calculate expected results on host
         for(size_t i = 0; i < output.size(); i++)
         {
-            T value = 0;
+            acc_type value(0);
             for(size_t j = 0; j < logical_warp_size; j++)
             {
                 auto idx = i * logical_warp_size + j;
-                value += input[idx];
+                value    = binary_op_host(input[idx], value);
             }
-            expected[i] = value;
+            expected[i] = static_cast<T>(value);
         }
 
         // Writing to device memory
@@ -222,12 +248,9 @@ TYPED_TEST(HipcubWarpReduceTests, Reduce)
             )
         );
 
-        for(size_t i = 0; i < output.size(); i++)
-        {
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
-        }
+        test_utils::assert_near(output,
+                                expected,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -268,6 +291,11 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
     // The different warp sizes
@@ -310,20 +338,20 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
         SCOPED_TRACE(testing::Message() << "with seed= " << seed_value);
 
         // Generate data
-        std::vector<T> input = test_utils::get_random_data<T>(size, -100, 100, seed_value);
-        std::vector<T> output(size / logical_warp_size, 0);
-        std::vector<T> expected(output.size(), 1);
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size / logical_warp_size);
+        std::vector<T> expected(output.size());
 
         // Calculate expected results on host
         for(size_t i = 0; i < output.size(); i++)
         {
-            T value = 0;
-            for(size_t j = 0; j < valid; j++)
+            acc_type value(0);
+            for(int j = 0; j < valid; ++j)
             {
                 auto idx = i * logical_warp_size + j;
-                value += input[idx];
+                value    = binary_op_host(input[idx], value);
             }
-            expected[i] = valid ? value : input[i];
+            expected[i] = valid ? static_cast<T>(value) : input[i];
         }
 
         // Writing to device memory
@@ -370,12 +398,9 @@ TYPED_TEST(HipcubWarpReduceTests, ReduceValid)
             )
         );
 
-        for(size_t i = 0; i < output.size(); i++)
-        {
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
-        }
+        test_utils::assert_near(output,
+                                expected,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_output));
@@ -414,6 +439,11 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     using flag_type = unsigned char;
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
@@ -509,21 +539,21 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
         // Calculate expected results on host
         std::vector<T> expected(output.size());
         size_t segment_head_index = 0;
-        T reduction = input[0];
+        acc_type       reduction(input[0]);
         for(size_t i = 0; i < output.size(); i++)
         {
             if(i%logical_warp_size == 0 || flags[i])
             {
-                expected[segment_head_index] = reduction;
+                expected[segment_head_index] = static_cast<T>(reduction);
                 segment_head_index = i;
                 reduction = input[i];
             }
             else
             {
-                reduction = reduction + input[i];
+                reduction = binary_op_host(input[i], reduction);
             }
         }
-        expected[segment_head_index] = reduction;
+        expected[segment_head_index] = static_cast<T>(reduction);
 
         // Launching kernel
         if (current_device_warp_size == ws32)
@@ -560,15 +590,19 @@ TYPED_TEST(HipcubWarpReduceTests, HeadSegmentedReduceSum)
         );
         HIP_CHECK(hipDeviceSynchronize());
 
+        std::vector<T> output_segment(output.size(), T(0));
+        std::vector<T> expected_segment(output.size(), T(0));
         for(size_t i = 0; i < output.size(); i++)
         {
             if(flags[i])
             {
-                auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-                if(std::is_integral<T>::value) diff = 0;
-                ASSERT_NEAR(output[i], expected[i], diff) << "where index = " << i;
+                output_segment[i]   = output[i];
+                expected_segment[i] = expected[i];
             }
         }
+        test_utils::assert_near(output_segment,
+                                expected_segment,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_flags));
@@ -609,6 +643,11 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
     HIP_CHECK(hipSetDevice(device_id));
 
     using T = typename TestFixture::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
     using flag_type = unsigned char;
     // logical warp side for warp primitive, execution warp size
     constexpr size_t logical_warp_size = TestFixture::warp_size;
@@ -701,10 +740,11 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
         HIP_CHECK(hipDeviceSynchronize());
 
         // Calculate expected results on host
-        std::vector<T> expected(output.size());
+        std::vector<T>      expected(output.size());
         std::vector<size_t> segment_indexes;
-        size_t segment_index = 0;
-        T reduction;
+        size_t              segment_index = 0;
+        acc_type            accumulator(0);
+        acc_type            reduction(0);
         for(size_t i = 0; i < output.size(); i++)
         {
             // single value segments
@@ -712,20 +752,20 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
             {
                 expected[i] = input[i];
                 segment_indexes.push_back(i);
-            }
-            else
+            } else
             {
                 segment_index = i;
-                reduction = input[i];
-                auto next = i + 1;
+                reduction     = input[i];
+                auto next     = i + 1;
                 while(next < output.size() && !flags[next])
                 {
-                    reduction = reduction + input[next];
+                    reduction = binary_op_host(input[next], reduction);
                     i++;
                     next++;
                 }
                 i++;
-                expected[segment_index] = reduction + input[i];
+                accumulator             = binary_op_host(reduction, input[i]);
+                expected[segment_index] = static_cast<T>(accumulator);
                 segment_indexes.push_back(segment_index);
             }
         }
@@ -765,13 +805,17 @@ TYPED_TEST(HipcubWarpReduceTests, TailSegmentedReduceSum)
         );
         HIP_CHECK(hipDeviceSynchronize());
 
+        std::vector<T> output_segment(segment_indexes.size());
+        std::vector<T> expected_segment(segment_indexes.size());
         for(size_t i = 0; i < segment_indexes.size(); i++)
         {
             auto index = segment_indexes[i];
-            auto diff = std::max<T>(std::abs(0.1f * expected[i]), T(0.01f));
-            if(std::is_integral<T>::value) diff = 0;
-            ASSERT_NEAR(output[index], expected[index], diff) << "where index = " << i;
+            output_segment[i]   = output[index];
+            expected_segment[i] = expected[index];
         }
+        test_utils::assert_near(output_segment,
+                                expected_segment,
+                                test_utils::precision<T>::value * logical_warp_size);
 
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_flags));
